#!/usr/bin/python2
# -*- coding: utf-8 -*-

import networkx as nx
import unittest
import random

from generator.make_random_graphs import *

class deleteEdgesTests(unittest.TestCase):
    def setUp(self):
        self.G = nx.Graph()
        self.G.add_nodes_from(range(7))
        self.G.add_edge(1, 2)
        self.G.add_edge(1, 3)
        self.G.add_edge(4, 0)
        self.G.add_edge(2, 3)
        self.G.add_edge(0, 5)
        self.G.add_edge(5, 4)
        self.G.add_edge(6, 4)
        self.G.add_edge(6, 5)
        self.G.add_edge(6, 0)

    def testDeleteRandomEdgesError(self):
        with self.assertRaises(IndexError):
            deleteRandomEdges(self.G, 4)

    def testDeleteRandomEdges(self):
        random.seed(3)
        deleteRandomEdges(self.G, 3)
        self.assertTrue(not self.G.has_edge(2,3) and not self.G.has_edge(4,5) and
                        not self.G.has_edge(0,6))

class connectTests(unittest.TestCase):
    def setUp(self):
        self.G = nx.Graph()
        self.G.add_nodes_from(range(5))
        self.G.add_edge(1, 2)
        self.G.add_edge(1, 3)
        self.G.add_edge(4, 0)
        self.G.add_edge(2, 3)

    def testMakeConnectedError(self):
        self.G.remove_edge(2,3)
        with self.assertRaises(IndexError):
            makeConnected(self.G)

    def testMakeConnected(self):
        random.seed(5)
        makeConnected(self.G)
        self.assertTrue(self.G.has_edge(0,1) and not self.G.has_edge(1, 3))

class denseGraphTests(unittest.TestCase):
    def testRandomDenseGraph1(self):
        random.seed(5)
        G=randomDenseGraph(5)
        self.assertEqual(len(G.edges()), 8)
        self.assertTrue(not G.has_edge(2,3) and not G.has_edge(3,4))

    def testRandomDenseGraph2(self):
        random.seed(3)
        G=randomDenseGraph(10)
        self.assertEqual(len(G.edges()), 38)

class spanningTreeTests(unittest.TestCase):
    def testRandomSpanningTree(self):
        random.seed(3)
        self.G = nx.Graph()
        self.G.add_nodes_from(range(6))
        randomSpanningTree(self.G)
        self.assertTrue(nx.is_connected(self.G))
        self.assertEqual(nx.degree(self.G), {0:3, 1:1, 2:2, 3:1, 4:2, 5:1})

class addEdgesTests(unittest.TestCase):
    def setUp(self):
        self.G = nx.Graph()
        self.G.add_nodes_from(range(6))
        for i in range(5):
            self.G.add_edge(i,5)

    def testAddRandomEdgesError(self):
        with self.assertRaises(IndexError):
            addRandomEdges(self.G, 11)

    def testAddRandomEdgesNeg(self):
        addRandomEdges(self.G, -2)
        self.assertEqual(nx.degree(self.G), {0:1, 1:1, 2:1, 3:1, 4:1, 5:5})

    def testAddRandomEdges1(self):
        random.seed(5)
        addRandomEdges(self.G, 5)
        self.assertEqual(nx.degree(self.G), {0:3, 1:4, 2:4, 3:2, 4:2, 5:5})
        
    def testAddRandomEdges2(self):
        random.seed(3)
        addRandomEdges(self.G, 10)
        self.assertEqual(nx.degree(self.G), {0:5, 1:5, 2:5, 3:5, 4:5, 5:5})

class randomSparseGraphTests(unittest.TestCase):
    def testRandomSparseGraphTree(self):
        random.seed(5)
        G=randomSparseGraph(5)
        self.assertEqual(len(G.edges()), 4)

    def testRandomSparseGraph2(self):
        random.seed(3)
        G=randomSparseGraph(50)
        self.assertEqual(len(G.edges()), 69)
        self.assertTrue(G.has_edge(0, 41) and G.has_edge(34, 45))


if __name__ == '__main__':
    unittest.main()
