#!/usr/bin/python2
# -*- coding: utf-8 -*-

from netmic.mic import main
import argparse

parser = argparse.ArgumentParser(description='Computes an IIS arc cover with the'
                                 ' given method.')
parser.add_argument('network', help='network file or folder')
parser.add_argument('outputName', help='prefix for all output files',
                    nargs='?', default='output')

parser.add_argument('-t', '--timelimit', help='set a timelimit for the computa'
                    'tion of cover (in seconds)', type=float, default=900) 
    
parser.add_argument('-rb', '--refinementBellman', help='invoke refinement '
                    'heuristic with bellman', action='store_true', dest='bell')
parser.add_argument('-rsa', '--refinementSimpleAssignment', help='invoke '
                    'refinement heuristic with Simple Assignment',
                    action='store_true', dest='simple')
parser.add_argument('-rma', '--refinementMultipleAssignment', help='invoke '
                    'refinement heuristic with Multiple Assignment',
                    action='store_true', dest='multi')
parser.add_argument('-m', '--minimalizer', help='invoke postprocessing step '
                    'minimalizer', action='store_true', dest='mini')
parser.add_argument('-tp', '--timelimitPostprocessing', help='set a timelimit for'
                    ' the computation of postprocessing (in seconds)', type=float,
                    dest='timePost', default=900) 

# choice of method to compute cover 
subparsers = parser.add_subparsers(help='choose method to solve IAC',dest='func')

parser_1_1 =  subparsers.add_parser('1.1', help='use heuristic 1.1')
parser_1_1.add_argument('-sp', '--shortestPath', help='which method for the '
                        'shortest path computations should be used',
                        choices=['dijkstra', 'floyd', 'combined'],
                        default='dijkstra')

parser_1_2 =  subparsers.add_parser('1.2', help='use heuristic 1.2')

parser_1_3 =  subparsers.add_parser('1.3', help='use heuristic 1.3')
parser_1_3.add_argument('-var', '--variant', help='choose second variant of SPFP',
                         action='store_true')

parser_2_2 =  subparsers.add_parser('2.2', help='use heuristic 2.2')

parser_2_3 =  subparsers.add_parser('2.3', help='use heuristic 2.3')
parser_2_3.add_argument('-var', '--variant', help='choose second variant of SPFP',
                         action='store_true')

parser_maxfs =  subparsers.add_parser('maxfs', help='use heuristic greedy maxFS')
parser_maxfs.add_argument('-b', '--blockSize', help='no. of inequalities tested '
                          'at once', type=int, default=10)

parser_ell1 =  subparsers.add_parser('ell1', help='solve ell_1 model')

parser_IISF =  subparsers.add_parser('IISFixed', help='use heuristic '
                                     'IISFixedCover')
parser_IISF.add_argument('-red', '--redundant', help='use also redundant IISs',
                         action='store_true')

parser_IISS =  subparsers.add_parser('IISSet', help='use heuristic IISSetCover')
parser_IISS.add_argument('-red', '--redundant', help='use also redundant IISs',
                         action='store_true')

parser_IISH =  subparsers.add_parser('IISHitting', help='use heuristic '
                                     'IISHitting')
parser_IISH.add_argument('-red', '--redundant', help='use also redundant IISs',
                         action='store_true')

parser_tree =  subparsers.add_parser('tree', help='use tree approximation')
parser_tree.add_argument('-w', '--weight', help='which weights should be used '
                         'for MST',
                         choices=['upper','uniform','negative','demand','factor'],
                         default='demand')

parser_treeAll =  subparsers.add_parser('treeAll', help='use tree approximation '
                                        'with all weight functions')

parser_FPT =  subparsers.add_parser('FPT', help='use elementary cycle '
                                    'enumeration')

parser_mip =  subparsers.add_parser('mip', help='solve optimal mip model')
parser_mip.add_argument('-s', '--startSolution', help='compute a lower bound and'
                        ' start solution', action='store_true', dest='start')

parser_setCover =  subparsers.add_parser('setCover', help='solve optimal set '
                                         'cover model')

args = parser.parse_args()
main(args)


