#!/usr/bin/python2
# -*- coding: utf-8 -*-

import networkx as nx
import unittest
import netmic.algorithmic_utility as alUtil

from netmic.heuristics.elementary_cycle_enumeration import *

class solverTest(unittest.TestCase):
    def setUp(self):
        self.G = nx.DiGraph()
        self.G.add_node(1, supply=3)
        self.G.add_node(2, supply=2)
        self.G.add_node(3, supply=-2)
        self.G.add_node(4, supply=-1)
        self.G.add_node(5, supply=-1)
        self.G.add_node(6, supply=-1)
        self.G.add_edge(1, 2, upper=2)
        self.G.add_edge(1, 3, upper=2)
        self.G.add_edge(3, 4, upper=1)
        self.G.add_edge(2, 3, upper=1)
        self.G.add_edge(3, 4, upper=1)
        self.G.add_edge(3, 5, upper=0)
        self.G.add_edge(6, 5, upper=2)

    def testEnumerateElementaryCycles(self):
        C,x,it= enumerateElementaryCycles(self.G)
        self.assertEqual(C, {1:{}, 2:{3:'u'}, 3:{5:'u'}, 4:{}, 5:{}, 6:{5:'l'}})
        self.assertEqual(it, 2)
        self.assertEqual(x, {1:{2:1, 3:2}, 2:{3:3}, 3:{4:1, 5:2}, 4:{}, 5:{},
                             6:{5:-1}})

    def testSolve(self):
        builder = elementaryCycleSolver(self.G)
        builder.solve()
        self.assertEqual(builder.x, {1: {2: 1, 3: 2}, 2: {3: 3}, 3: {4: 1, 5: 2},
                                     4: {}, 5: {}, 6: {5: -1}})

    def testCheckChangeToBound(self):
        builder = elementaryCycleSolver(self.G)
        cycle = [1,3,2,1]
        self.assertEqual(builder.checkChangeToBound(1,cycle), 0)
        self.assertEqual(builder.checkChangeToBound(-1,cycle), 1)
        
    def testImproved(self):
        builder = elementaryCycleSolver(self.G)
        self.assertEqual(builder.improved(1,2,1), 0)
        self.assertEqual(builder.improved(2,3,-1), 1)
        self.assertEqual(builder.improved(3,4,2), -1)
        self.assertEqual(builder.improved(2,3,-3), 0)
        
    def testStartSolution(self):
        builder = elementaryCycleSolver(self.G)
        self.assertEqual(builder.x, {1: {2: 0, 3: 3}, 2: {3: 2}, 3: {4: 1, 5: 2},
                                     4: {}, 5: {}, 6: {5: -1}})
       
if __name__ == '__main__':
    unittest.main()
