#!/usr/bin/python2
# -*- coding: utf-8 -*-

import networkx as nx
import unittest
import pulp

from netmic.heuristics.ell1_model import *

class Ell1ModelTests(unittest.TestCase):
    def setUp(self):
        self.G = nx.DiGraph()
        self.G.add_node(1, supply=3)
        self.G.add_node(2, supply=2)
        self.G.add_node(3, supply=-2)
        self.G.add_node(4, supply=-1)
        self.G.add_node(5, supply=-1)
        self.G.add_node(6, supply=-1)
        self.G.add_edge(1, 2, upper=2)
        self.G.add_edge(1, 3, upper=2)
        self.G.add_edge(2, 3, upper=1)
        self.G.add_edge(3, 4, upper=2)
        self.G.add_edge(3, 5, upper=0)
        self.G.add_edge(6, 5, upper=2)

    def testBuildModel(self):
        modelBuilder = ell1Model(self.G)
        self.assertEqual(modelBuilder.lp.numVariables(), 18)
        self.assertEqual(modelBuilder.lp.numConstraints(), 18)
        self.assertEqual(modelBuilder.lp.getSense(), 1)
        self.assertEqual(sum(1 for v in modelBuilder.lp.variables() if
                             v.cat==pulp.LpInteger), 0)
        
    def testBuildModelWithSolve(self):
        modelBuilder = ell1Model(self.G)
        modelBuilder.lp.solve()
        self.assertEqual(modelBuilder.x[(6,5)].varValue, -1)
        self.assertEqual(modelBuilder.x[(3,5)].varValue, 2)
        self.assertEqual(modelBuilder.x[(2,3)].varValue, 3)
        self.assertEqual(pulp.value(modelBuilder.lp.objective), 5)
        
    @unittest.skipUnless(pulp.CPLEX.available(pulp.CPLEX()),
                         "buildCover() requires CPLEX")
    def testBuildCover(self):
        modelBuilder = ell1Model(self.G)
        cover,x = modelBuilder.buildCover()
        self.assertEqual(cover, {1:{3:'u'}, 2:{3:'u'}, 3:{5:'u'}, 4:{}, 5:{},
                                 6:{5:'l'}})
        self.assertEqual(x, {1:{2:0.0, 3:3.0}, 2:{3:2.0}, 3:{4:1.0, 5:2.0}, 4:{},
                             5:{}, 6:{5:-1.0}})
        
if __name__ == '__main__':
    unittest.main()
