#!/usr/bin/python2
# -*- coding: utf-8 -*-

import networkx as nx
import unittest
import netmic.algorithmic_utility as alUtil
import netmic.network_utility as netUtil

from netmic.heuristics.weight_functions import *


class algorithmTest(unittest.TestCase):
    def setUp(self):
        self.G = nx.DiGraph()
        self.G.add_node(1, supply=3)
        self.G.add_node(2, supply=2)
        self.G.add_node(3, supply=-2)
        self.G.add_node(4, supply=-1)
        self.G.add_node(5, supply=-1)
        self.G.add_node(6, supply=-1)
        self.G.add_edge(1, 2, upper=2)
        self.G.add_edge(1, 3, upper=2)
        self.G.add_edge(2, 3, upper=1)
        self.G.add_edge(3, 4, upper=2)
        self.G.add_edge(3, 5, upper=0)
        self.G.add_edge(6, 5, upper=2)
        self.x1 = {1: {3: 2}, 2: {3: 1}, 3: {4: 1}}
        alUtil.completeSolution(self.G, self.x1)
        self.x2 = {1: {3: 3}, 2: {3: 1}, 3: {4: 1, 5: 1}}
        alUtil.completeSolution(self.G, self.x2)

    def testComputeArcWeightsSimple(self):
        auxG = netUtil.arcPairNetwork(self.G)
        computeArcWeightsSimple(self.G, auxG, self.x2, auxG.edges_iter())
        sumWeights = sum(a[2]["weight"] for a in auxG.edges_iter(data=True))
        self.assertEqual(sumWeights, 13)

        computeArcWeightsSimple(self.G, auxG, self.x1,
                                netUtil.arcsOnPath([1,3,5,6]))
        sumWeights = sum(a[2]["weight"] for a in auxG.edges_iter(data=True))
        self.assertEqual(sumWeights, 18)

    def testComputeArcWeightsDetailedBinary(self):
        auxG = netUtil.arcPairNetwork(self.G)
        computeArcWeightsDetailedBinary(self.G, auxG, self.x2, 1)
        sumWeights = sum(a[2]["weight"] for a in auxG.edges_iter(data=True))
        self.assertEqual(sumWeights, 3)

    def testComputeArcWeightsDetailedTernary(self):
        auxG = netUtil.arcPairNetwork(self.G)
        computeArcWeightsDetailedTernary(self.G, auxG, self.x2, 1)
        sumWeights = sum(a[2]["weight"] for a in auxG.edges_iter(data=True))
        self.assertEqual(sumWeights, 1)

    def testComputeArcWeightsDetailedTernaryBug(self):
        auxG = netUtil.arcPairNetwork(self.G)
        x = {1:{2: -1, 3: 4}, 2:{3: 1}, 3:{4: 1, 5: 2}, 4:{}, 5:{}, 6:{5: -1}}
        computeArcWeightsDetailedTernary(self.G, auxG, x, 2)
        self.assertEqual(auxG[3][1]['weight'], -1)
        
if __name__ == '__main__':
    unittest.main()
