#!/usr/bin/python2
# -*- coding: utf-8 -*-

import networkx as nx
import unittest
import pulp

from netmic.optimal.setCover import *

class SetCoverTests(unittest.TestCase):
    def setUp(self):
        self.G = nx.DiGraph()
        self.G.add_node(1, supply=3)
        self.G.add_node(2, supply=2)
        self.G.add_node(3, supply=-2)
        self.G.add_node(4, supply=-1)
        self.G.add_node(5, supply=-1)
        self.G.add_node(6, supply=-1)
        self.G.add_edge(1, 2, upper=2)
        self.G.add_edge(1, 3, upper=2)
        self.G.add_edge(2, 3, upper=1)
        self.G.add_edge(3, 4, upper=2)
        self.G.add_edge(3, 5, upper=0)
        self.G.add_edge(6, 5, upper=2)

    def testResetBoundsInitially(self):
        ICB = setCover(self.G)
        ICB._setBoundsToInf([(6,5),(3,5)])
        ICB._resetBoundsInitially()
        self.assertEqual(ICB.ssN[6][5]['upper'], 2)
        self.assertFalse(ICB.ssN.has_edge(5,6))
        self.assertEqual(ICB._R[3][5]['capacity'], 0)

    def testSetupMip(self):
        ICB = setCover(self.G)
        ICB.IISList = [{'delta': [(3,4), (3,5)], 'covered': 0},
                       {'delta': [(3,5)], 'covered': 0}]
        ICB._setUpMip()
        self.assertEqual(ICB.mip.numVariables(), 2)
        self.assertEqual(sum(1 for v in ICB.mip.variables() if
                             v.cat==pulp.LpInteger), 6)
        self.assertEqual(ICB.mip.numConstraints(), 2)
        self.assertEqual(ICB.mip.getSense(), 1)

    def testUpdateMip(self):
        ICB = setCover(self.G)
        ICB.IISList = [{'delta': [(3,4), (3,5)], 'covered': 0},
                       {'delta': [(3,5)], 'covered': 0}]
        ICB._setUpMip()
        ICB.listOfS = [[1,2]]
        ICB._updateMip()
        self.assertEqual(ICB.mip.numVariables(), 4)
        self.assertEqual(ICB.mip.numConstraints(), 3)

    @unittest.skipUnless(pulp.CPLEX.available(pulp.CPLEX()),
                         "_solveOptimally() requires CPLEX")
    def testSolveOptimally(self):
        ICB = setCover(self.G)
        ICB.IISList = [{'delta': [(3,4), (3,5)], 'covered': 0},
                       {'delta': [(3,5)], 'covered': 0}]
        ICB._setUpMip()
        ICB._solveOptimally()
        self.assertFalse(ICB.listOfS)
        self.assertEqual(ICB.y[(3,5)].varValue, 1)
        self.assertEqual(ICB.y[(2,3)].varValue, 1)
        self.assertEqual(ICB.y[(6,5)].varValue, 1)
        self.assertEqual(pulp.value(ICB.mip.objective), 3)

    @unittest.skipUnless(pulp.CPLEX.available(pulp.CPLEX()),
                         "_solveOptimally() requires CPLEX")
    def testSolveViaIISSetCover(self):
        C,it,_,_ = solveViaIISSetCover(self.G)
        self.assertEqual(C, {1:{}, 2:{3:'u'}, 3:{5:'u'}, 4:{}, 5:{},
                             6:{5:'l'}})
        self.assertEqual(it, 2)
                             
if __name__ == '__main__':
    unittest.main()
