#!/usr/bin/python2
# -*- coding: utf-8 -*-

import networkx as nx
import unittest

from netmic.postprocessing.minimalize import *

class minimalizerTests(unittest.TestCase):                
    def setUp(self):
        self.G = nx.DiGraph()
        self.G.add_node(1, supply=3)
        self.G.add_node(2, supply=2)
        self.G.add_node(3, supply=-2)
        self.G.add_node(4, supply=-1)
        self.G.add_node(5, supply=-1)
        self.G.add_node(6, supply=-1)
        self.G.add_edge(1, 2, upper=2)
        self.G.add_edge(1, 3, upper=2)
        self.G.add_edge(2, 3, upper=1)
        self.G.add_edge(3, 4, upper=2)
        self.G.add_edge(3, 5, upper=0)
        self.G.add_edge(6, 5, upper=2)
        self.C = {1:{2:'l', 3:'u'}, 2:{}, 3:{5:'u'}, 4:{}, 5:{}, 6:{5:'l'}}

    def testMinimizeSame(self):
        self.assertEqual(minimalize(self.G, self.C), self.C)
 
    def testMinimized(self):
        self.C[2][3] = 'u' 
        self.assertEqual(minimalize(self.G, self.C), {1:{}, 2:{3:'u'}, 3:{5:'u'},
                                                      4:{}, 5:{}, 6:{5:'l'}})
    def testCleanUp(self):
        builder = minimalizer(self.G)
        builder.check(self.C)
        builder.cleanUp()
        self.assertEqual(sum(builder._R[w][v]['capacity'] for v,w in
                             self.G.edges_iter()), 0)
        self.assertEqual(sum(builder._R[v][w]['capacity'] for v,w in
                             self.G.edges_iter()), 9)
        
    def testCheckFalse(self):
        builder = minimalizer(self.G)
        self.assertFalse(builder.check(self.C))
        
    def testCheckTrue(self):
        self.C[2][3] = 'u' 
        builder = minimalizer(self.G)
        self.assertEqual(builder.check(self.C), 2)
        
    def testRemoveCoverBoundsInitially(self):
        builder = minimalizer(self.G)
        builder.C = self.C
        builder._removeCoverBoundsInitially()
        self.assertEqual(builder._R[2][1]['capacity'], 57)
        self.assertEqual(builder._R[3][5]['capacity'], 57)
        self.assertEqual(builder.ssN[1][2]['upper'], 2)

    def testAddZeroBoundsToR(self):
        builder = minimalizer(self.G)
        self.assertTrue(builder._R.has_edge(3,5) and builder._R.has_edge(5,3))

        
if __name__ == '__main__':
    unittest.main()
