#!/usr/bin/python2
# -*- coding: utf-8 -*-

import networkx as nx
import unittest
import netmic.algorithmic_utility as alUtil
import netmic.network_utility as netUtil

from netmic.postprocessing.refinement import *

class refinementTests(unittest.TestCase):
    def setUp(self):
        self.G = nx.DiGraph()
        self.G.add_node(1, supply=3)
        self.G.add_node(2, supply=2)
        self.G.add_node(3, supply=-2)
        self.G.add_node(4, supply=-1)
        self.G.add_node(5, supply=-1)
        self.G.add_node(6, supply=-1)
        self.G.add_edge(1, 2, upper=2)
        self.G.add_edge(1, 3, upper=2)
        self.G.add_edge(2, 3, upper=1)
        self.G.add_edge(3, 4, upper=2)
        self.G.add_edge(3, 5, upper=0)
        self.G.add_edge(6, 5, upper=2)
        self.x1 = {1: {3: 2}, 2: {3: 1}, 3: {4: 1}}
        alUtil.completeSolution(self.G, self.x1)
        self.x2 = {1: {2: -1, 3: 4}, 2: {3: 1}, 3: {4: 1, 5: 2}, 6: {5:-1}}
        alUtil.completeSolution(self.G, self.x2)

    def testRefineSolution1(self):
        refineSolution(self.G, self.x1)
        self.assertFalse(refineSolution(self.G, self.x1))

    def testRefineSolution2(self):
        self.assertTrue(refineSolution(self.G, self.x2))
        self.assertEqual(self.x2, {1: {2: 2, 3: 1}, 2: {3: 4}, 3: {4: 1, 5: 2},
                                   4:{}, 5:{}, 6: {5:-1}})

    def testNegCyclesSimple(self):
        revArcsN = netUtil.arcPairNetwork(self.G)
        for v,w in revArcsN.edges_iter():
            revArcsN.edge[v][w]['weight'] = -1
        cycles = negCycles(revArcsN, 'simpleAss')
        self.assertEqual(cycles, [])

    def testNegCyclesMulti(self):
        revArcsN = netUtil.arcPairNetwork(self.G)
        for v,w in revArcsN.edges_iter():
            revArcsN.edge[v][w]['weight'] = -1
        cycles = negCycles(revArcsN, 'multiAss')
        self.assertEqual(cycles, [[1,3,2,1]])

    def testNegCyclesBellman(self):
        revArcsN = netUtil.arcPairNetwork(self.G)
        for v,w in revArcsN.edges_iter():
            revArcsN.edge[v][w]['weight'] = -1
        cycles = negCycles(revArcsN, 'bellman')
        self.assertEqual(cycles, [[1,2,3,1]])
                             
if __name__ == '__main__':
    unittest.main()
