# -*- coding: utf-8 -*-
'''
Preprocessing functions.
'''
#   Copyright (C) 2015 by
#   Imke Joormann <joormann@mathematik.tu-darmstadt.de>
#   All rights reserved.
#   BSD license.

import networkx as nx
import netmic.network_utility as netUtil
import netmic.algorithmic_utility as alUtil

def preprocessDegreeOneNodes(N):
    '''Remove nodes v with deg(v) = 1 from N.

    Decide optimally if $\delta(v)$ is in the cover and adds it accordingly;
    remove node v and $\delta(v)$, add b(v) to b(w) where w is the neighbour of v.
    Assumes l = 0.

    Args: N: network problem 
    Returns: Partial cover 
    '''
    C = alUtil.emptySolutionDict(N)
    nodeList = netUtil.listDegreeOneNodes(N)
    
    i = 0   # iterate by index since list is changed inside the loop
    while i < len(nodeList) and len(N) > 1:
        v = nodeList[i]
        if len(N.successors(v)) == 1:
            neighbour = N.successors(v)[0]
        elif len(N.predecessors(v)) == 1:
            neighbour = N.predecessors(v)[0]
        else:
            raise ValueError("Expected degree one node")        

        if netUtil.GHSupplyViolated(N, [v]):
            if N.has_edge(v, neighbour):
                C[v][neighbour] = "u"
            elif N.has_edge(neighbour, v):
                C[neighbour][v] = "l"
        elif netUtil.GHDemandViolated(N, [v]):
            if N.has_edge(v, neighbour):
                C[v][neighbour] = "l"
            elif N.has_edge(neighbour, v):
                C[neighbour][v] = "u"
        
        N.node[neighbour]['supply'] += N.node[v]['supply']
        N.remove_node(v)

        if N.degree(neighbour) == 1: # a new deg-1-node came into existence
            nodeList.append(neighbour)            
        i += 1
        
    return C


def removeDegreeOneInnodes(N):
    '''Removes all nodes v with b(v)=0 and deg(v)=1 from N.'''
    innodeList = netUtil.listOfInNodes(N)
    i = 0   # iterate by index since list is changed inside the loop
    while i < len(innodeList):
        v = innodeList[i]
        nbour = netUtil.allNeighbours(N, v)
        if len(nbour) == 1:
            N.remove_node(v) 
            if N.node[nbour[0]]["supply"] == 0: # a new deg-1-node came into
                innodeList.append(nbour[0])     # existence
        i += 1

        
