#!/usr/bin/python2
# -*- coding: utf-8 -*-

import networkx as nx
import unittest
import netmic.algorithmic_utility as alUtil
from netmic.preprocessing import *

################################################
class preprocessDegreeOneNodesTests(unittest.TestCase):
    def setUp(self):
        self.G = nx.DiGraph()
        self.G.add_node(1, supply=3)
        self.G.add_node(2, supply=2)
        self.G.add_node(3, supply=-2)
        self.G.add_node(4, supply=-1)
        self.G.add_node(5, supply=-1)
        self.G.add_node(6, supply=-1)
        self.G.add_edge(1, 2, upper=2)
        self.G.add_edge(1, 3, upper=2)
        self.G.add_edge(2, 3, upper=1)
        self.G.add_edge(3, 4, upper=2)
        self.G.add_edge(3, 5, upper=0)
        self.G.add_edge(6, 5, upper=2)

    def testPreprocessDegreeOneNode(self):
        C = alUtil.emptySolutionDict(self.G)
        C[3][5] = "u"
        C[6][5] = "l"
        self.assertDictEqual(preprocessDegreeOneNodes(self.G), C)
        self.assertEqual(len(self.G), 3)
        self.assertEqual(self.G.node[3]['supply'], -5)

class preprocessDegreeOneNodesCompleteTests(unittest.TestCase):
    def setUp(self):
        self.G = nx.DiGraph()
        self.G.add_node(0, supply=2)
        self.G.add_node(1, supply=0)
        self.G.add_node(2, supply=-2)        
        self.G.add_edge(1, 0, upper=1, lower=-float('inf'))
        self.G.add_edge(1, 2, upper=1)

    def testDegreeOneNodeFully(self):
        preprocessDegreeOneNodes(self.G)
        self.assertEqual(self.G.nodes(), [1])
    

        
class removeDegreeOneNodesTests(unittest.TestCase):
    def setUp(self): 
        self.G = nx.DiGraph()
        self.G.add_node(1, supply=1)
        self.G.add_node(2, supply=-1)
        self.G.add_node(3, supply=0)
        self.G.add_node(4, supply=0)
        self.G.add_node(5, supply=0)
        self.G.add_edge(1, 2, lower=1, upper=2)
        self.G.add_edge(1, 3, lower=2, upper=2)
        self.G.add_edge(3, 4, lower=1, upper=4)
        self.G.add_edge(4, 5, lower=1, upper=4)
        
    def testDegreeOneNodeRemoval(self):
        removeDegreeOneInnodes(self.G)
        self.assertEqual(nx.number_of_nodes(self.G), 2)
        self.assertFalse(self.G.has_node(3) or self.G.has_node(4)  or
                         self.G.has_node(5))
        

if __name__ == '__main__':
    unittest.main()
