#!/usr/bin/python2
# -*- coding: utf-8 -*-

import networkx as nx
import unittest

from netmic.solution_output import *

class CoverTests(unittest.TestCase):
    def setUp(self):
        self.G = nx.DiGraph()
        self.G.add_node(2, supply=2)
        self.G.add_node(3, supply=-2)
        self.G.add_node(4, supply=-1)
        self.G.add_node(5, supply=-1)
        self.G.add_node(6, supply=-1)
        self.G.add_edge(1, 2, upper=2)
        self.G.add_edge(1, 3, upper=2)
        self.G.add_edge(2, 3, upper=1)
        self.G.add_edge(3, 4, upper=2)
        self.G.add_edge(3, 5, upper=0)
        self.G.add_edge(6, 5, upper=2)
        
        self.C = dict([(v, {}) for v in self.G])
        self.C[2][3] = "u"
        self.C[3][5] = "u"
        self.C[6][5] = "l"

    def testListOfUpperViolatedArcs(self):
        self.assertListEqual(listOfUpperViolatedArcs(self.C),
                             [(2,3), (3,5)])

    def testListOfLowerViolatedArcs(self):
        self.assertListEqual(listOfLowerViolatedArcs(self.C),
                             [(6,5)])

    def testCoverToFeasibleNetwork(self):
        coverToFeasibleNetwork(self.G, [(1,2), (2,3)], [(3,5)])
        self.assertEqual(self.G[1][2]['upper'], float('inf'))
        self.assertEqual(self.G[3][5]['lower'], float('-inf'))

if __name__ == '__main__':
    unittest.main()
